@@ -7,11 +7,12 @@ See [Line API](/guide/widgets#line-api) for how to use Strips.
 
 from __future__ import annotations
 
-from itertools import chain
-from typing import Iterable, Iterator, Sequence
+from functools import lru_cache
+from typing import Any, Iterable, Iterator, Sequence
 
 import rich.repr
 from rich.cells import cell_len, set_cell_size
+from rich.color import ColorSystem
 from rich.console import Console, ConsoleOptions, RenderResult
 from rich.measure import Measurement
 from rich.segment import Segment
@@ -20,10 +21,11 @@ from rich.style import Style, StyleType
 from textual._segment_tools import index_to_cell_position, line_pad
 from textual.cache import FIFOCache
 from textual.color import Color
-from textual.constants import DEBUG
 from textual.css.types import AlignHorizontal, AlignVertical
 from textual.filter import LineFilter
 
+SGR_STYLES = ["1", "2", "3", "4", "5", "6", "7", "8", "9", "21", "51", "52", "53"]
+
 
 def get_line_length(segments: Iterable[Segment]) -> int:
     """Get the line length (total length of all segments).
@@ -39,7 +41,7 @@ def get_line_length(segments: Iterable[Segment]) -> int:
 
 
 class StripRenderable:
-    """A renderable which renders a list of strips in to lines."""
+    """A renderable which renders a list of strips into lines."""
 
     def __init__(self, strips: list[Strip], width: int | None = None) -> None:
         self._strips = strips
@@ -84,7 +86,9 @@ class Strip:
         "_render_cache",
         "_line_length_cache",
         "_crop_extend_cache",
+        "_offsets_cache",
         "_link_ids",
+        "_cell_count",
     ]
 
     def __init__(
@@ -104,16 +108,17 @@ class Strip:
             tuple[int, int, Style | None],
             Strip,
         ] = FIFOCache(4)
+        self._offsets_cache: FIFOCache[tuple[int, int], Strip] = FIFOCache(4)
         self._render_cache: str | None = None
         self._link_ids: set[str] | None = None
-
-        if DEBUG and cell_length is not None:
-            # If `cell_length` is incorrect, render will be fubar
-            assert get_line_length(self._segments) == cell_length
+        self._cell_count: int | None = None
 
     def __rich_repr__(self) -> rich.repr.Result:
-        yield self._segments
-        yield self.cell_length
+        try:
+            yield self._segments
+            yield self.cell_length
+        except AttributeError:
+            pass
 
     @property
     def text(self) -> str:
@@ -164,7 +169,7 @@ class Strip:
         strips: list[Strip],
         style: Style,
         width: int,
-        height: int,
+        height: int | None,
         horizontal: AlignHorizontal,
         vertical: AlignVertical,
     ) -> Iterable[Strip]:
@@ -202,18 +207,19 @@ class Strip:
                 yield blank
 
         top_blank_lines = bottom_blank_lines = 0
-        vertical_excess_space = max(0, height - shape_height)
+        if height is not None:
+            vertical_excess_space = max(0, height - shape_height)
 
-        if vertical == "top":
-            bottom_blank_lines = vertical_excess_space
-        elif vertical == "middle":
-            top_blank_lines = vertical_excess_space // 2
-            bottom_blank_lines = vertical_excess_space - top_blank_lines
-        elif vertical == "bottom":
-            top_blank_lines = vertical_excess_space
+            if vertical == "top":
+                bottom_blank_lines = vertical_excess_space
+            elif vertical == "middle":
+                top_blank_lines = vertical_excess_space // 2
+                bottom_blank_lines = vertical_excess_space - top_blank_lines
+            elif vertical == "bottom":
+                top_blank_lines = vertical_excess_space
 
-        if top_blank_lines:
-            yield from blank_lines(top_blank_lines)
+            if top_blank_lines:
+                yield from blank_lines(top_blank_lines)
 
         if horizontal == "left":
             for strip in strips:
@@ -276,7 +282,7 @@ class Strip:
 
     @classmethod
     def join(cls, strips: Iterable[Strip | None]) -> Strip:
-        """Join a number of strips in to one.
+        """Join a number of strips into one.
 
         Args:
             strips: An iterable of Strips.
@@ -284,16 +290,24 @@ class Strip:
         Returns:
             A new combined strip.
         """
+        join_strips = [
+            strip for strip in strips if strip is not None and strip.cell_count
+        ]
+        segments = [segment for strip in join_strips for segment in strip._segments]
+        cell_length: int | None = None
+        if any([strip._cell_length is None for strip in join_strips]):
+            cell_length = None
+        else:
+            cell_length = sum([strip._cell_length or 0 for strip in join_strips])
+        joined_strip = cls(segments, cell_length)
+        if all(strip._render_cache is not None for strip in join_strips):
+            joined_strip._render_cache = "".join(
+                [strip._render_cache for strip in join_strips]
+            )
+        return joined_strip
 
-        segments: list[list[Segment]] = []
-        add_segments = segments.append
-        total_cell_length = 0
-        for strip in strips:
-            if strip is not None:
-                total_cell_length += strip.cell_length
-                add_segments(strip._segments)
-        strip = cls(chain.from_iterable(segments), total_cell_length)
-        return strip
+    def __add__(self, other: Strip) -> Strip:
+        return Strip.join([self, other])
 
     def __bool__(self) -> bool:
         return not not self._segments  # faster than bool(...)
@@ -308,10 +322,22 @@ class Strip:
         return len(self._segments)
 
     def __eq__(self, strip: object) -> bool:
-        return isinstance(strip, Strip) and (
-            self._segments == strip._segments and self.cell_length == strip.cell_length
+        return isinstance(strip, Strip) and (self._segments == strip._segments)
+
+    def __getitem__(self, index: int | slice) -> Strip:
+        if isinstance(index, int):
+            index = slice(index, index + 1)
+        return self.crop(
+            index.start, self.cell_count if index.stop is None else index.stop
         )
 
+    @property
+    def cell_count(self) -> int:
+        """Number of cells in the strip"""
+        if self._cell_count is None:
+            self._cell_count = sum(len(segment.text) for segment in self._segments)
+        return self._cell_count
+
     def extend_cell_length(self, cell_length: int, style: Style | None = None) -> Strip:
         """Extend the cell length if it is less than the given value.
 
@@ -340,6 +366,9 @@ class Strip:
             A new strip with the supplied cell length.
         """
 
+        if self.cell_length == cell_length:
+            return self
+
         cache_key = (cell_length, style)
         cached_strip = self._line_length_cache.get(cache_key)
         if cached_strip is not None:
@@ -382,7 +411,7 @@ class Strip:
         return strip
 
     def simplify(self) -> Strip:
-        """Simplify the segments (join segments with same style)
+        """Simplify the segments (join segments with same style).
 
         Returns:
             New strip.
@@ -393,6 +422,34 @@ class Strip:
         )
         return line
 
+    def discard_meta(self) -> Strip:
+        """Remove all meta from segments.
+
+        Returns:
+            New strip.
+        """
+
+        def remove_meta_from_segment(segment: Segment) -> Segment:
+            """Build a Segment with no meta.
+
+            Args:
+                segment: Segment.
+
+            Returns:
+                Segment, sans meta.
+            """
+            text, style, control = segment
+            if style is None:
+                return segment
+            style = style.copy()
+            style._meta = None
+            return Segment(text, style, control)
+
+        return Strip(
+            [remove_meta_from_segment(segment) for segment in self._segments],
+            self._cell_length,
+        )
+
     def apply_filter(self, filter: LineFilter, background: Color) -> Strip:
         """Apply a filter to all segments in the strip.
 
@@ -516,7 +573,7 @@ class Strip:
         return strip
 
     def divide(self, cuts: Iterable[int]) -> Sequence[Strip]:
-        """Divide the strip in to multiple smaller strips by cutting at given (cell) indices.
+        """Divide the strip into multiple smaller strips by cutting at given (cell) indices.
 
         Args:
             cuts: An iterable of cell positions as ints.
@@ -529,8 +586,7 @@ class Strip:
         cell_length = self.cell_length
         cuts = [cut for cut in cuts if cut <= cell_length]
         cache_key = tuple(cuts)
-        cached = self._divide_cache.get(cache_key)
-        if cached is not None:
+        if (cached := self._divide_cache.get(cache_key)) is not None:
             return cached
 
         strips: list[Strip]
@@ -564,6 +620,90 @@ class Strip:
         self._style_cache[style] = styled_strip
         return styled_strip
 
+    def apply_meta(self, meta: dict[str, Any]) -> Strip:
+        """Apply meta to all segments.
+
+        Args:
+            meta: A dict of meta information.
+
+        Returns:
+            A new strip.
+
+        """
+        meta_style = Style.from_meta(meta)
+        return self.apply_style(meta_style)
+
+    def _apply_link_style(self, link_style: Style) -> Strip:
+        segments = self._segments
+        _Segment = Segment
+        segments = [
+            (
+                _Segment(
+                    text,
+                    (
+                        style
+                        if style._meta is None
+                        else (style + link_style if "@click" in style.meta else style)
+                    ),
+                    control,
+                )
+                if style
+                else _Segment(text)
+            )
+            for text, style, control in segments
+        ]
+        return Strip(segments, self._cell_length)
+
+    @classmethod
+    @lru_cache(maxsize=16384)
+    def render_ansi(cls, style: Style, color_system: ColorSystem) -> str:
+        """Render ANSI codes for a give style.
+
+        Args:
+            style: A Rich style.
+            color_system: Color system enumeration.
+
+        Returns:
+            A string of ANSI escape sequences to render the style.
+        """
+        sgr: list[str]
+        if attributes := style._attributes & style._set_attributes:
+            _style_map = SGR_STYLES
+            sgr = [
+                _style_map[bit_offset]
+                for bit_offset in range(attributes.bit_length())
+                if attributes & (1 << bit_offset)
+            ]
+        else:
+            sgr = []
+        if (color := style._color) is not None:
+            sgr.extend(color.downgrade(color_system).get_ansi_codes())
+        if (bgcolor := style._bgcolor) is not None:
+            sgr.extend(bgcolor.downgrade(color_system).get_ansi_codes(False))
+        ansi = style._ansi = ";".join(sgr)
+        return ansi
+
+    @classmethod
+    def render_style(cls, style: Style, text: str, color_system: ColorSystem) -> str:
+        """Render a Rich style and text.
+
+        Args:
+            style: Style to render.
+            text: Content string.
+            color_system: Color system enumeration.
+
+        Returns:
+            Text with ANSI escape sequences.
+        """
+        if (ansi := style._ansi) is None:
+            ansi = cls.render_ansi(style, color_system)
+        output = f"\x1b[{ansi}m{text}\x1b[0m" if ansi else text
+        if style._link:
+            output = (
+                f"\x1b]8;id={style._link_id};{style._link}\x1b\\{output}\x1b]8;;\x1b\\"
+            )
+        return output
+
     def render(self, console: Console) -> str:
         """Render the strip into terminal sequences.
 
@@ -574,15 +714,19 @@ class Strip:
             Rendered sequences.
         """
         if self._render_cache is None:
-            color_system = console._color_system
-            render = Style.render
+            color_system = console._color_system or ColorSystem.TRUECOLOR
+            render = self.render_style
             self._render_cache = "".join(
                 [
-                    render(style, text, color_system=color_system)
+                    (
+                        text
+                        if style is None
+                        else render(style, text, color_system=color_system)
+                    )
                     for text, style, _ in self._segments
-                    if style is not None
                 ]
             )
+
         return self._render_cache
 
     def crop_pad(self, cell_length: int, left: int, right: int, style: Style) -> Strip:
@@ -609,3 +753,64 @@ class Strip:
         if right:
             segments.append(Segment(" " * right, style))
         return Strip(segments, cell_length + left + right)
+
+    def text_align(self, width: int, align: AlignHorizontal) -> Strip:
+        if align == "left":
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(self._segments, 0, width - self.cell_length, Style.null()),
+                    width,
+                )
+        elif align == "center":
+            left_space = max(0, width - self.cell_length) // 2
+
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(
+                        self._segments,
+                        left_space,
+                        width - self.cell_length - left_space,
+                        Style.null(),
+                    ),
+                    width,
+                )
+
+        elif align == "right":
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(self._segments, width - self.cell_length, 0, Style.null()),
+                    width,
+                )
+
+    def apply_offsets(self, x: int, y: int) -> Strip:
+        """Apply offsets used in text selection.
+
+        Args:
+            x: Offset on X axis (column).
+            y: Offset on Y axis (row).
+
+        Returns:
+            New strip.
+        """
+        cache_key = (x, y)
+        if (cached_strip := self._offsets_cache.get(cache_key)) is not None:
+            return cached_strip
+        segments = self._segments
+        strip_segments: list[Segment] = []
+        for segment in segments:
+            text, style, _ = segment
+            offset_style = Style.from_meta({"offset": (x, y)})
+            strip_segments.append(
+                Segment(text, style + offset_style if style else offset_style)
+            )
+            x += len(segment.text)
+        strip = Strip(strip_segments, self._cell_length)
+        strip._render_cache = self._render_cache
+        self._offsets_cache[cache_key] = strip
+        return strip
